@@ -38,6 +38,8 @@ def codegen(
     examples: Optional[str] = None,
     package: Optional[str] = None,
     copyright: Optional[str] = None,
+    spdx_copyright_text: Optional[List[str]] = None,
+    spdx_license_identifier: Optional[str] = None,
     parser_info: Optional[str] = None,
 ) -> None:
     """Generate classes with loaders for the given Schema Salad description."""
@@ -59,7 +61,7 @@ def codegen(
     info = parser_info or pkg
     salad_version = schema_metadata.get("saladVersion", "v1.1")
 
-    if lang in set(["python", "cpp", "dlang"]):
+    if lang in {"python", "cpp", "dlang"}:
         if target:
             dest: Union[TextIOWrapper, TextIO] = open(target, mode="w", encoding="utf-8")
         else:
@@ -71,6 +73,8 @@ def codegen(
                 examples,
                 pkg,
                 copyright,
+                spdx_copyright_text,
+                spdx_license_identifier,
             )
             gen.parse(j)
             return
@@ -110,8 +114,12 @@ def codegen(
     document_roots = []
 
     for rec in j:
-        if rec["type"] in ("enum", "record"):
-            gen.type_loader(rec)
+        if rec["type"] in ("enum", "map", "record", "union"):
+            jld = rec.get("jsonldPredicate")
+            if isinstance(jld, MutableMapping):
+                gen.type_loader(rec, jld.get("_container"), jld.get("noLinkCheck"))
+            else:
+                gen.type_loader(rec)
             gen.add_vocab(shortname(rec["name"]), rec["name"])
 
     for rec in j:
@@ -171,12 +179,14 @@ def codegen(
 
             for field in sorted_fields:
                 optional = bool("https://w3id.org/cwl/salad#null" in field["type"])
-                type_loader = gen.type_loader(field["type"])
                 jld = field.get("jsonldPredicate")
                 fieldpred = field["name"]
                 subscope = None
 
                 if isinstance(jld, MutableMapping):
+                    type_loader = gen.type_loader(
+                        field["type"], jld.get("_container"), jld.get("noLinkCheck")
+                    )
                     ref_scope = jld.get("refScope")
                     subscope = jld.get("subscope")
                     if jld.get("typeDSL"):
@@ -185,10 +195,16 @@ def codegen(
                         type_loader = gen.secondaryfilesdsl_loader(type_loader)
                     elif jld.get("_type") == "@id":
                         type_loader = gen.uri_loader(
-                            type_loader, jld.get("identity", False), False, ref_scope
+                            type_loader,
+                            jld.get("identity", False),
+                            False,
+                            ref_scope,
+                            jld.get("noLinkCheck"),
                         )
                     elif jld.get("_type") == "@vocab":
-                        type_loader = gen.uri_loader(type_loader, False, True, ref_scope)
+                        type_loader = gen.uri_loader(
+                            type_loader, False, True, ref_scope, jld.get("noLinkCheck")
+                        )
 
                     map_subject = jld.get("mapSubject")
                     if map_subject:
@@ -201,6 +217,8 @@ def codegen(
 
                     if "_id" in jld and jld["_id"][0] != "@":
                         fieldpred = jld["_id"]
+                else:
+                    type_loader = gen.type_loader(field["type"])
 
                 if jld == "@id":
                     continue
