@@ -4,6 +4,7 @@ import os
 import pathlib
 import re
 import tempfile
+import traceback
 import urllib
 import xml.sax  # nosec
 from io import StringIO
@@ -50,7 +51,6 @@ from .utils import (
 )
 
 _logger = logging.getLogger("salad")
-typeDSLregex = re.compile(r"^([^[?]+)(\[\])?(\?)?$")
 
 
 def file_uri(path: str, split_frag: bool = False) -> str:
@@ -140,6 +140,7 @@ def SubLoader(loader: "Loader") -> "Loader":
         url_fields=loader.url_fields,
         allow_attachments=loader.allow_attachments,
         session=loader.session,
+        salad_version=loader.salad_version,
     )
 
 
@@ -157,6 +158,7 @@ class Loader:
         url_fields: Optional[Set[str]] = None,
         allow_attachments: Optional[AttachmentsType] = None,
         doc_cache: Union[str, bool] = True,
+        salad_version: Optional[str] = None,
     ) -> None:
         self.idx: IdxType = (
             NormDict(lambda url: urllib.parse.urlsplit(url).geturl()) if idx is None else idx
@@ -206,6 +208,11 @@ class Loader:
         self.secondaryFile_dsl_fields: Set[str] = set()
         self.allow_attachments = allow_attachments
 
+        if salad_version:
+            self.salad_version = salad_version
+        else:
+            self.salad_version = "v1.1"
+
         self.add_context(ctx)
 
     def expand_url(
@@ -290,7 +297,9 @@ class Loader:
                 try:
                     content = self.fetch_text(fetchurl)
                 except Exception as e:
+                    tb = traceback.format_exception(type(e), e, e.__traceback__)
                     _logger.warning("Could not load extension schema %s: %s", fetchurl, str(e))
+                    _logger.debug(tb)
                     continue
                 newGraph = Graph()
                 err_msg = "unknown error"
@@ -458,15 +467,13 @@ class Loader:
                         break
                 if not lref:
                     raise ValidationException(
-                        "Object `{}` does not have identifier field in {}".format(
-                            obj, self.identifiers
-                        ),
+                        f"Object {obj!r} does not have identifier field in {self.identifiers}",
                         SourceLine(obj),
                     )
 
         if not isinstance(lref, str):
             raise ValidationException(
-                f"Expected CommentedMap or string, got {type(lref)}: `{lref}`"
+                f"Expected CommentedMap or string, got {type(lref)}: {lref!r}"
             )
 
         if isinstance(lref, str) and os.sep == "\\":
@@ -489,7 +496,7 @@ class Loader:
                         return resolved_obj, metadata
                 else:
                     raise ValidationException(
-                        f"Expected CommentedMap, got {type(metadata)}: `{metadata}`"
+                        f"Expected CommentedMap, got {type(metadata)}: {metadata!r}"
                     )
             elif isinstance(resolved_obj, MutableSequence):
                 metadata = self.idx.get(urllib.parse.urldefrag(url)[0], CommentedMap())
@@ -501,9 +508,8 @@ class Loader:
                 return resolved_obj, CommentedMap()
             else:
                 raise ValidationException(
-                    "Expected MutableMapping or MutableSequence, got {}: `{}`".format(
-                        type(resolved_obj), resolved_obj
-                    )
+                    f"Expected MutableMapping or MutableSequence, got "
+                    f"{type(resolved_obj)}: {resolved_obj!r}"
                 )
 
         # "$include" directive means load raw text
@@ -524,9 +530,8 @@ class Loader:
                 # If the base document is in the index, it was already loaded,
                 # so if we didn't find the reference earlier then it must not
                 # exist.
-                raise ValidationException(
-                    f"Reference `#{frg}` not found in file `{doc_url}`.",
-                    SourceLine(self.idx[doc_url]),
+                raise SourceLine(self.idx, doc_url, ValidationException).makeError(
+                    f"Reference '#{frg}' not found in file {doc_url!r}.",
                 )
             doc = self.fetch(doc_url, inject_ids=(not mixin), content_types=content_types)
 
@@ -562,9 +567,7 @@ class Loader:
                 resolved_obj = self.idx[url]
             else:
                 raise ValidationException(
-                    "Reference `{}` is not in the index. Index contains: {}".format(
-                        url, ", ".join(self.idx)
-                    )
+                    f"Reference {url!r} is not in the index. Index contains: {' '.join(self.idx)}"
                 )
 
         if isinstance(resolved_obj, CommentedMap):
@@ -605,8 +608,8 @@ class Loader:
                                 v.lc.filename = document.lc.filename
                             else:
                                 raise ValidationException(
-                                    "mapSubject '{}' value '{}' is not a dict "
-                                    "and does not have a mapPredicate.".format(k, v),
+                                    f"mapSubject {k!r} value {v!r} is not a dict "
+                                    "and does not have a mapPredicate.",
                                     SourceLine(document, idmapField),
                                 )
                         else:
@@ -634,23 +637,41 @@ class Loader:
         if not isinstance(t, str):
             return t
 
-        m = typeDSLregex.match(t)
-        if not m:
-            return t
-        first = m.group(1)
-        assert first  # nosec
-        second = third = None
-        if bool(m.group(2)):
-            second = CommentedMap((("type", "array"), ("items", first)))
-            second.lc.add_kv_line_col("type", lc)
-            second.lc.add_kv_line_col("items", lc)
-            second.lc.filename = filename
-        if bool(m.group(3)):
-            third = CommentedSeq(["null", second or first])
-            third.lc.add_kv_line_col(0, lc)
-            third.lc.add_kv_line_col(1, lc)
-            third.lc.filename = filename
-        return third or second or first
+        t_ = t
+        optional = False
+        if t_.endswith("?"):
+            optional = True
+            t_ = t_[0:-1]
+
+        if t_.endswith("[]"):
+            salad_versions = [int(v) for v in self.salad_version[1:].split(".")]
+            rest = t_[0:-2]
+            if salad_versions < [1, 3]:
+                if rest.endswith("[]"):
+                    # To show the error message with the original type
+                    return t
+                else:
+                    cmap = CommentedMap((("type", "array"), ("items", rest)))
+            else:
+                items = self._type_dsl(rest, lc, filename)
+                cmap = CommentedMap((("type", "array"), ("items", items)))
+            cmap.lc.add_kv_line_col("type", lc)
+            cmap.lc.add_kv_line_col("items", lc)
+            cmap.lc.filename = filename
+            expanded: Union[str, CommentedMap, CommentedSeq] = cmap
+        else:
+            expanded = t_
+
+        if optional:
+            cs = CommentedSeq(["null", expanded])
+            cs.lc.add_kv_line_col(0, lc)
+            cs.lc.add_kv_line_col(1, lc)
+            cs.lc.filename = filename
+            ret: Union[str, CommentedMap, CommentedSeq] = cs
+        else:
+            ret = expanded
+
+        return ret
 
     def _secondaryFile_dsl(
         self,
@@ -679,10 +700,9 @@ class Loader:
     ) -> Union[str, CommentedMap, CommentedSeq]:
         if d in loader.type_dsl_fields:
             return self._type_dsl(datum, lc, filename)
-        elif d in loader.secondaryFile_dsl_fields:
+        if d in loader.secondaryFile_dsl_fields:
             return self._secondaryFile_dsl(datum, lc, filename)
-        else:
-            return datum
+        return datum
 
     def _resolve_dsl(
         self,
@@ -833,7 +853,7 @@ class Loader:
                     checklinks=checklinks,
                     strict_foreign_properties=strict_foreign_properties,
                 )
-            elif "$mixin" in document:
+            if "$mixin" in document:
                 return self.resolve_ref(
                     document,
                     base_url=base_url,
@@ -1034,9 +1054,7 @@ class Loader:
         if onWindows() and link.startswith("file:"):
             link = link.lower()
         raise ValidationException(
-            "Field `{}` references unknown identifier `{}`, tried {}".format(
-                field, link, ", ".join(tried)
-            )
+            f"Field {field!r} references unknown identifier {link!r}, tried {' '.join(tried)}"
         )
 
     def validate_link(
@@ -1057,14 +1075,14 @@ class Loader:
                         return self.validate_scoped(field, link, docid)
                     elif not self.check_exists(link):
                         raise ValidationException(
-                            f"Field `{field}` contains undefined reference to `{link}`"
+                            f"Field {field!r} contains undefined reference to {link!r}"
                         )
             elif link not in self.idx and link not in self.rvocab:
                 if field in self.scoped_ref_fields:
                     return self.validate_scoped(field, link, docid)
                 elif not self.check_exists(link):
                     raise ValidationException(
-                        f"Field `{field}` contains undefined reference to `{link}`"
+                        f"Field {field!r} contains undefined reference to {link!r}"
                     )
         elif isinstance(link, CommentedSeq):
             errors = []
@@ -1134,7 +1152,7 @@ class Loader:
                             and sl.makeLead() != all_doc_ids[document[identifier]]
                         ):
                             _logger.warning(
-                                "%s object %s `%s` previously defined",
+                                "%s object %s %r previously defined",
                                 all_doc_ids[document[identifier]],
                                 identifier,
                                 relname(document[identifier]),
@@ -1165,18 +1183,17 @@ class Loader:
                     docid2 = self.getid(val)
                     if docid2 is not None:
                         errors.append(
-                            ValidationException(f"checking object `{relname(docid2)}`", sl, [v])
+                            ValidationException(f"checking object {relname(docid2)!r}", sl, [v])
                         )
                     else:
                         if isinstance(key, str):
-                            errors.append(ValidationException(f"checking field `{key}`", sl, [v]))
+                            errors.append(ValidationException(f"checking field {key!r}", sl, [v]))
                         else:
                             errors.append(ValidationException("checking item", sl, [v]))
         if bool(errors):
             if len(errors) > 1:
                 raise ValidationException("", None, errors)
-            else:
-                raise errors[0]
+            raise errors[0]
         return
 
 
