@@ -52,7 +52,7 @@ def reflow_all(text: str, maxline: Optional[int] = None) -> str:
         assert group is not None  # nosec
         maxno = max(maxno, len(group))
     maxno_text = maxline - maxno
-    msg = []  # type: List[str]
+    msg: List[str] = []
     for line in text.splitlines():
         g = lineno_re.match(line)
         if not g:
@@ -85,8 +85,7 @@ def indent(v: str, nolead: bool = False, shift: str = "  ", bullet: str = "  ")
         return v.splitlines()[0] + "\n".join([shift + line for line in v.splitlines()[1:]])
 
     def lineno(i: int, line: str) -> str:
-        r = lineno_re.match(line)
-        if r is not None:
+        if (r := lineno_re.match(line)) is not None:
             group1 = r.group(1)
             group2 = r.group(2)
             assert group1 is not None  # nosec
@@ -109,7 +108,7 @@ def strip_duplicated_lineno(text: str) -> str:
 
     Same as :py:meth:`strip_dup_lineno` but without reflow.
     """
-    pre = None  # type: Optional[str]
+    pre: Optional[str] = None
     msg = []
     for line in text.splitlines():
         g = lineno_re.match(line)
@@ -131,7 +130,7 @@ def strip_duplicated_lineno(text: str) -> str:
 def strip_dup_lineno(text: str, maxline: Optional[int] = None) -> str:
     if maxline is None:
         maxline = int(os.environ.get("COLUMNS", "100"))
-    pre = None  # type: Optional[str]
+    pre: Optional[str] = None
     msg = []
     maxno = 0
     for line in text.splitlines():
